local *

{
  :AcceptQuest
  :C_GossipInfo
  :CompleteQuest
  :GetNumActiveQuests
  :GetNumAvailableQuests
  :GetNumQuestChoices
  :GetNumQuestLogEntries
  :GetQuestLogSelection
  :GetQuestLogTitle
  :IsAltKeyDown
  :IsControlKeyDown
  :IsQuestCompletable
  :IsShiftKeyDown
  :QuestFrameCompleteQuestButton
  :SelectActiveQuest
  :SelectAvailableQuest
  :SelectQuestLogEntry
  :GossipTitleButton1
  :GossipTitleButton2
  :ShowQuestComplete
  :UnitName
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :SelectOption
  :ToggleOption
} = idTweaksOptions

class Questing extends idTweaksModule
  new: (addon) =>
    -- print 'Questing#new'

    super addon, 'Questing', 'AceConsole-3.0', 'AceEvent-3.0'
    
    @completed_quests = {}
    @incomplete_quests = {}

  set_options: =>
    -- print 'Questing#set_options'

    super!

    @options\add_option SelectOption @, 'pause_key', 'Pause Key', 'While you hold this key nothing will happen', {
      values:
        shift: 'Shift'
        control: 'Control'
        alt: 'Alt'
    }, {}, 'shift'

    @options\add_option ToggleOption @, 'accept_quests', 'Accept Quests', 'Accept quests'
    @options\add_option ToggleOption @, 'complete_quests', 'Complete Quests', 'Automatically complete quests unless when offered a choice of rewards'
    @options\add_option ToggleOption @, 'complete_gossip', 'Complete Gossip', 'When the gossip window has one option, use it'
    @options\add_option ToggleOption @, 'complete_covenant_sanctum', 'Complete Seat of the Primus', 'Automate interactions in the Seat of the Primus'

  enable: =>
    -- print 'Questing#enable'

    @ace_module\RegisterEvent 'GOSSIP_SHOW', (...) -> @GOSSIP_SHOW ...
    @ace_module\RegisterEvent 'QUEST_AUTOCOMPLETE', (...) -> @QUEST_AUTOCOMPLETE ...
    @ace_module\RegisterEvent 'QUEST_COMPLETE', (...) -> @QUEST_COMPLETE ...
    @ace_module\RegisterEvent 'QUEST_DETAIL', (...) -> @QUEST_DETAIL ...
    @ace_module\RegisterEvent 'QUEST_GREETING', (...) -> @QUEST_GREETING ...
    @ace_module\RegisterEvent 'QUEST_LOG_UPDATE', (...) -> @QUEST_LOG_UPDATE ...
    @ace_module\RegisterEvent 'QUEST_PROGRESS', (...) -> @QUEST_PROGRESS ...

  disable: =>
    -- print 'Questing#disable'

    @ace_module\UnregisterEvent 'GOSSIP_SHOW'
    @ace_module\UnregisterEvent 'QUEST_AUTOCOMPLETE'
    @ace_module\UnregisterEvent 'QUEST_COMPLETE'
    @ace_module\UnregisterEvent 'QUEST_DETAIL'
    @ace_module\UnregisterEvent 'QUEST_GREETING'
    @ace_module\UnregisterEvent 'QUEST_LOG_UPDATE'
    @ace_module\UnregisterEvent 'QUEST_PROGRESS'

  should_automate: =>
    -- print 'Questing#should_automate'

    output = true
    key = @db.pause_key

    if key == 'shift'
      if IsShiftKeyDown!
        output = false

    if key == 'control'
      if IsControlKeyDown!
        output = false

    if key == 'alt'
      if IsAltKeyDown!
        output = false

    output

  should_accept: =>
    -- print 'Questing#should_accept'

    return @db.accept_quests

  should_complete: =>
    -- print 'Questing#should_complete'

    return @db.complete_quests

  strip_text: (text) =>
    -- print 'Questing#strip_text'

    return unless text

    text = text\gsub '|c%x%x%x%x%x%x%x%x(.-)|r', '%1'
    text = text\gsub '%[.*%]%s*', ''
    text = text\gsub '(.+) %(.+%)', '%1'
    text = text\trim!

    text
    
  index_quests: =>
    -- print 'Questing#index_quests'

    @completed_quests = {}
    @incomplete_quests = {}
    
    num_entries = GetNumQuestLogEntries!

    if num_entries > 1
      for i = 1, num_entries
        SelectQuestLogEntry i
        title, _, _, _, _, is_complete = GetQuestLogTitle i
        
        if title
          if is_complete
            @completed_quests[title] = true
          else
            @incomplete_quests[title] = true

  autocomplete_quest: (questId) =>
    -- print 'Questing#autocomplete_quest'

    return unless @should_complete!

    ShowQuestComplete questId

  progress_quest: () =>
    -- print 'Questing#progress_quest'

    return unless @should_complete!

    if IsQuestCompletable!
      CompleteQuest!

  complete_quest: () =>
    -- print 'Questing#complete_quest'

    return unless @should_complete!

    if GetNumQuestChoices! <= 1
      QuestFrameCompleteQuestButton\Click!

  accept_quest: () =>
    -- print "Questing#accept_quest"

    return unless @should_accept!

    AcceptQuest!

  automate_gossip_window: () =>
    -- print 'Questing#automate_gossip_window'
    
    return unless @should_automate!
    
    if idTweaks\is_bc!
      -- If there is one gossip option, use it.
      if GossipTitleButton1\IsVisible! and (GossipTitleButton1.type == 'Gossip' or GossipTitleButton1.type == 'Active') and not GossipTitleButton2\IsVisible!
        GossipTitleButton1\Click!

      -- Complete Quests
      for i = 32, 1, -1
        button = _G["QuestTitleButton#{i}"]
        if button and button\IsVisible!
          text = @strip_text button\GetText!
          if @completed_quests[text] and @should_complete!
            button\Click!
          elseif @incomplete_quests[text] and @should_accept!
            button\Click!

      -- Complete Quests (Gossip Button)
      for i = 32, 1, -1
        button = _G["GossipTitleButton#{i}"]
        if button and button\IsVisible!
          text = @strip_text button\GetText!
          if @completed_quests[text] and @should_complete!
            button\Click!
          elseif @incomplete_quests[text] and @should_accept!
            button\Click!

    if idTweaks\is_retail!
      -- In the new API, complete quests and then accept them. You always get all progress in one click.
      num_active_quests = C_GossipInfo.GetNumActiveQuests!
      -- print "Questing#automate_gossip_window:num_active_quests #{num_active_quests}"
      if num_active_quests > 0
        for i = 1, num_active_quests
          C_GossipInfo.SelectActiveQuest i

      num_available_quests = C_GossipInfo.GetNumAvailableQuests!
      -- print "Questing#automate_gossip_window:num_available_quests #{num_available_quests}"
      if num_available_quests > 0
        for i = 1, num_available_quests
          C_GossipInfo.SelectAvailableQuest i

      -- In the old API, accept quests before completing quests. You might have to click multiple times.
      num_available_quests = GetNumAvailableQuests!
      -- print "Questing#automate_gossip_window:num_available_quests #{num_available_quests}"
      if num_available_quests > 0
        for i = 1, num_available_quests
          SelectAvailableQuest i

      num_active_quests = GetNumActiveQuests!
      -- print "Questing#automate_gossip_window:num_active_quests #{num_active_quests}"
      if num_active_quests > 0
        for i = 1, num_active_quests
          SelectActiveQuest i

      num_gossips = C_GossipInfo.GetNumOptions!
      -- print "Questing#automate_gossip_window:num_gossips #{num_gossips}"
      if num_gossips == 1
        C_GossipInfo.SelectOption 1

  QUEST_AUTOCOMPLETE: (event, questId) =>
    -- print 'Questing#QUEST_AUTOCOMPLETE', event, questId

    return unless @should_automate!

    @autocomplete_quest(questId)

  QUEST_COMPLETE: (event) =>
    -- print 'Questing#QUEST_COMPLETE'

    return unless @should_automate!

    @complete_quest!

  QUEST_DETAIL: =>
    -- print 'Questing#QUEST_DETAIL'

    return unless @should_automate!

    @accept_quest!

  QUEST_GREETING: (...) =>
    -- print 'Questing#QUEST_GREETING'

    return unless @should_automate!

    @automate_gossip_window!

  QUEST_PROGRESS: =>
    -- print 'Questing#QUEST_PROGRESS'

    return unless @should_automate!

    @accept_quest!
    
  QUEST_LOG_UPDATE: =>
    -- print 'Questing#QUEST_LOG_UPDATE'
    
    return unless idTweaks\is_bc!
    return unless @should_automate!
    
    @index_quests!
    
    start_entry = GetQuestLogSelection!

    SelectQuestLogEntry start_entry

idTweaks\add_module Questing idTweaks
